@@ -1,4 +1,4 @@
-/*	$Id: YSM_Commands.c,v 1.86 2004/05/29 21:27:19 rad2k Exp $	*/
+/*	$Id: YSM_Commands.c,v 1.94 2005/11/14 01:58:28 rad2k Exp $	*/
 /*
 -======================== ysmICQ client ============================-
 		Having fun with a boring Protocol
@@ -27,7 +27,7 @@ For Contact information read the AUTHORS file.
 */
 
 #include "YSM.h"
-__RCSID("$Id: YSM_Commands.c,v 1.86 2004/05/29 21:27:19 rad2k Exp $");
+__RCSID("$Id: YSM_Commands.c,v 1.94 2005/11/14 01:58:28 rad2k Exp $");
 
 #include "YSM_Lists.h"
 #include "YSM_Commands.h"
@@ -543,10 +543,14 @@ u_int32_t	fprint_bkp = 0;
 
 		} else {
 
+			int8_t	*msgCol = "";
+			if (YSMSlaves_Comfortable->Color != NULL)
+				msgCol = YSMSlaves_Comfortable->Color;
+
 			PRINTF(VERBOSE_BASE,
-			"Switching to %s -comfortable- message Mode.\n"
+			"%sSwitching to %s -comfortable- message Mode.\n"
 			"Use '.' in an empty line to end Message or '#'"
-			"to cancel the whole Message.\n", 
+			"to cancel the whole Message.\n", msgCol,
 			((YSM_Query != NULL) && !plainflag
 			&& (!YSM_KeyEmpty(YSMSlaves_Comfortable->crypto.strkey))
 			&& (YSMSlaves_Comfortable->fprint == FINGERPRINT_YSM_CLIENT_CRYPT)) 
@@ -762,14 +766,19 @@ char	*aux;
 	if (YSMSlaves_LastSent) {
 
 		if (!argc) {
+			int8_t	*msgCol="";
 			YSMSlaves_Comfortable = YSMSlaves_LastSent;
 
+			if (YSMSlaves_Comfortable->Color != NULL)
+				msgCol=YSMSlaves_Comfortable->Color;
+			
+
 			PRINTF(VERBOSE_BASE,
-			"Switching to -comfortable- message "
-			"Mode. (sending %s to " WHITE"%s" NORMAL")",
+			"%sSwitching to -comfortable- message "
+			"Mode. (sending %s to " WHITE"%s" NORMAL")%s", msgCol,
 			(!YSM_KeyEmpty(YSMSlaves_Comfortable->crypto.strkey)) 
 			? "ENCRYPTED" : "PLAIN",
-			YSMSlaves_Comfortable->info.NickName);
+			YSMSlaves_Comfortable->info.NickName, msgCol);
 
 			PRINTF(VERBOSE_BASE,
 			"\nUse '.' in an empty line to end Message or '#' "
@@ -817,15 +826,20 @@ char	*aux = NULL;
 	if (YSMSlaves_LastRead) {
 
 		if (!argc) {
+			int8_t	*msgCol="";
 			YSMSlaves_Comfortable = YSMSlaves_LastRead;
 			
+			if (YSMSlaves_Comfortable->Color != NULL)
+				msgCol=YSMSlaves_Comfortable->Color;
+			
+			
 			PRINTF(VERBOSE_BASE,
-			"Switching to -comfortable- message "
-			"Mode. (sending %s to "WHITE"%s" NORMAL")",
+			"%sSwitching to -comfortable- message "
+			"Mode. (sending %s to "WHITE"%s" NORMAL")%s", msgCol,
 			((!YSM_KeyEmpty(YSMSlaves_Comfortable->crypto.strkey)) 
 			&& (YSMSlaves_Comfortable->fprint == FINGERPRINT_YSM_CLIENT_CRYPT)) 
 			? "ENCRYPTED" : "PLAIN",
-			YSMSlaves_Comfortable->info.NickName);
+			YSMSlaves_Comfortable->info.NickName, msgCol);
 
 			PRINTF(VERBOSE_BASE,
 			"\nUse '.' in an empty line to end Message or '#' "
@@ -1082,7 +1096,7 @@ YSM_Command_READAFK( int argc, char **argv )
 	PRINTF( VERBOSE_BASE, "%s\n", MSG_AFK_READ_MSG);
 
 	PRE_GETKEY_FIX
-	YSM_ReadLog(YSM_AFKFILENAME);
+	YSM_ReadLog(YSM_AFKFILENAME, 0);
 	POS_GETKEY_FIX
 }
 
@@ -1180,8 +1194,13 @@ static void
 YSM_Command_RENAME( int argc, char **argv ) 
 {
 YSM_SLAVE	*YSM_Query = NULL;
+int8_t		*newslavename = NULL;
 
-	YSM_Query = YSM_QuerySlaves(SLAVE_NAME, argv[2], 0, 0);
+	newslavename = argv[2];
+	/* sanitize the slave name, trim? */
+	while (*newslavename == 0x20) newslavename++;
+
+	YSM_Query = YSM_QuerySlaves(SLAVE_NAME, newslavename, 0, 0);
 
 	if (YSM_Query) {
 		/* The new name exists! Abort! */
@@ -1203,7 +1222,7 @@ YSM_SLAVE	*YSM_Query = NULL;
 
 	/* The old_name exists, renaming a valid slave */
 
-	if (!strcasecmp(YSM_Query->info.NickName, argv[2])) {
+	if (!strcasecmp(YSM_Query->info.NickName, newslavename)) {
 		/* Renaming to the same name? no way! */
 		PRINTF( VERBOSE_BASE,
 			"Ahahaha..thats a joke, right? Renaming"
@@ -1214,9 +1233,9 @@ YSM_SLAVE	*YSM_Query = NULL;
 	PRINTF( VERBOSE_BASE, 
 		"Renaming %s to %s\n",
 		YSM_Query->info.NickName,
-		argv[2]);
+		newslavename);
 
-	YSM_UpdateSlave( UPDATE_NICK, argv[2], YSM_Query->Uin );
+	YSM_UpdateSlave( UPDATE_NICK, newslavename, YSM_Query->Uin );
 }
 
 static void
@@ -1514,7 +1533,7 @@ static void
 YSM_Command_HIST( int argc, char **argv ) 
 {
 YSM_SLAVE	*YSM_Query = NULL;
-char		UinStr[MAX_UIN_LEN];
+char		UinStr[MAX_UIN_LEN+1];
 
 	YSM_Query = YSM_QuerySlaves(SLAVE_NAME, argv[1], 0, 0);
 	if (!YSM_Query) {
@@ -1529,11 +1548,11 @@ char		UinStr[MAX_UIN_LEN];
 		"about the 'log' command.\n",
 		YSM_Query->info.NickName, YSM_Query->Uin);
 
-	snprintf(UinStr, MAX_UIN_LEN, "%d", YSM_Query->Uin );
+	snprintf(UinStr, MAX_UIN_LEN, "%d", (int)YSM_Query->Uin );
 	UinStr[sizeof(UinStr)-1] = 0x00;
 
 	PRE_GETKEY_FIX
-	YSM_ReadLog( UinStr ); 
+	YSM_ReadLog(UinStr, 1); 
 	POS_GETKEY_FIX
 }
 
@@ -1831,6 +1850,12 @@ YSM_SLAVE	*YSM_Query = NULL;
 static void
 YSM_Command_PASSWORD( int argc, char **argv ) 
 {
+	if (strlen(argv[1]) < 4 || strlen(argv[1]) > 8) {
+		// icq passwords can only be between 4 and 8 characters.
+		PRINTF(VERBOSE_BASE, "Incorrect password length. Password must be between 4 and 8 characters long.\n");
+		return;
+	}
+
 	YSM_ChangePassword( argv[1] );
 }
 
@@ -1876,7 +1901,7 @@ int8_t		*data = NULL, tmp[MAX_UIN_LEN + MAX_NICK_LEN + 3], am[3];
 			snprintf( tmp,
 				MAX_UIN_LEN + MAX_NICK_LEN + 2,
 				"%d%c%s%c",
-				YSM_Query->Uin,
+				(int)YSM_Query->Uin,
 				0xFE,
 				YSM_Query->info.NickName,
 				0xFE );
@@ -1903,7 +1928,7 @@ int8_t		*data = NULL, tmp[MAX_UIN_LEN + MAX_NICK_LEN + 3], am[3];
 		snprintf( tmp,
 			MAX_UIN_LEN + MAX_NICK_LEN + 2,
 			"%d%c%s%c",
-			list[y]->Uin,
+			(int)list[y]->Uin,
 			0xFE,
 			list[y]->info.NickName,
 			0xFE );
@@ -1913,7 +1938,7 @@ int8_t		*data = NULL, tmp[MAX_UIN_LEN + MAX_NICK_LEN + 3], am[3];
 	}	
 
 	/* Make the amount ASCII */
-	snprintf( am, sizeof(am), "%d", y );
+	snprintf( am, sizeof(am), "%d", (int)y );
 	am[sizeof(am) - 1] = 0x00;
 
 	/* Send them out! */
@@ -1922,6 +1947,7 @@ int8_t		*data = NULL, tmp[MAX_UIN_LEN + MAX_NICK_LEN + 3], am[3];
 	PRINTF( VERBOSE_BASE, "\rContacts sent.\n" );
 
 	YSM_Free( data, __FILE__, __LINE__ );
+	data = NULL;
 }
 
 
@@ -2244,6 +2270,14 @@ YSM_Command_SLAVESALL( int argc, char **argv )
 				0x0 );
 }
 
+#ifdef YSM_TRACE_MEMLEAK
+static void
+YSM_Command_SHOWLEAK( int argc, char **argv )
+{
+	PRINTF(VERBOSE_BASE, "Unfreed blocks: %d\n", unfreed_blocks);
+}
+#endif
+
 void
 YSM_Init_Commands( void )
 {
@@ -2723,6 +2757,18 @@ YSM_Init_Commands( void )
 			0,
 			&YSM_Command_LOADCONFIG);
 
+
+#ifdef YSM_TRACE_MEMLEAK
+	YSM_AddCommandtoList( "showleak",
+			NULL,
+			YSM_COMMAND_HELP_HELP,
+			YSM_COMMAND_GROUP_SETTINGS,
+			0,
+			&YSM_Command_SHOWLEAK );
+
+#endif
+
+
 }
 
 YSM_COMMAND *
