@@ -1,4 +1,4 @@
-/*	$Id: YSM_Config.h,v 1.1 2003/02/22 17:19:47 rad2k Exp $ 
+/*	$Id: YSM_Config.h,v 1.3 2005/11/14 01:59:11 rad2k Exp $ 
  *
  * YSM pre-compiling configuration.
  * ********************************
@@ -28,4 +28,10 @@
  *********************************************************/
 #undef YSM_WAR_MODE
 
+/* YSM_TRACE_MEMLEAK:
+ * enables the showleak command to help trace memory leaks.
+ *********************************************************/
+#undef YSM_TRACE_MEMLEAK
+
+
 #endif /* #ifndef _YSMCONFIGH_ */
