@@ -1,4 +1,4 @@
-/*	$Id: YSM_Prompt.c,v 1.186 2004/07/10 19:21:33 rad2k Exp $	*/
+/*	$Id: YSM_Prompt.c,v 1.202 2006/01/02 00:14:14 rad2k Exp $	*/
 /*
 -======================== ysmICQ client ============================-
 		Having fun with a boring Protocol
@@ -26,7 +26,7 @@ For Contact information read the AUTHORS file.
 */
 
 #include "YSM.h"
-__RCSID("$Id: YSM_Prompt.c,v 1.186 2004/07/10 19:21:33 rad2k Exp $");
+__RCSID("$Id: YSM_Prompt.c,v 1.202 2006/01/02 00:14:14 rad2k Exp $");
 
 #include "YSM_Lists.h"
 #include "YSM_Network.h"
@@ -121,7 +121,7 @@ int8_t status[MAX_STATUS_LEN];
 			sizeof(g_prompt),
 			"\r%s%-2.2s" NORMAL 
 			" " BRIGHT_BLUE "AFK[%d]" NORMAL "> ",
-			YSM_GetColorStatus(status),
+			YSM_GetColorStatus(status, NULL),
 			status,
 			YSM_AFKCount 
 			);
@@ -133,7 +133,7 @@ int8_t status[MAX_STATUS_LEN];
 		snprintf( g_prompt,
 			sizeof(g_prompt),
 			"\r%s[%d]" NORMAL "> ",
-			YSM_GetColorStatus(status),
+			YSM_GetColorStatus(status, NULL),
 			YSM_AFKCount);
 		g_prompt[sizeof(g_prompt)-1] = 0x00;
 	} else {
@@ -141,7 +141,7 @@ int8_t status[MAX_STATUS_LEN];
 		snprintf( g_prompt,
 			sizeof(g_prompt),
 			"\r%s%-2.2s" NORMAL "> ",
-			YSM_GetColorStatus(status),
+			YSM_GetColorStatus(status, NULL),
 			status
 			);
 		g_prompt[sizeof(g_prompt)-1] = 0x00;
@@ -152,23 +152,24 @@ int8_t status[MAX_STATUS_LEN];
 void
 YSM_ConsoleRedrawPrompt(int8_t redrawbuf)
 {
-	/* common for all, redraw the prompt */
-	PRINTF(VERBOSE_BASE, "\r%s", YSM_ConsoleGetPrompt());
-
 	if (redrawbuf) {
 #ifndef YSM_WITH_THREADS
-		if (strlen(g_cmdstring))
-			PRINTF(VERBOSE_BASE, "%s", g_cmdstring);
+		PRINTF(VERBOSE_BASE, 
+			"\r%s%s",
+			YSM_ConsoleGetPrompt(), 
+			g_cmdstring);
 #else
 #ifdef HAVE_LIBREADLINE
-		if (strlen(rl_line_buffer))
-			PRINTF(VERBOSE_BASE, "%s", rl_line_buffer);
+		PRINTF(VERBOSE_BASE, 
+			"\r%s%s",
+			YSM_ConsoleGetPrompt(), 
+			rl_line_buffer);
 #else
-		if (strlen(gl_buf))
-			PRINTF(VERBOSE_BASE, "%s", gl_buf);
+		gl_fixup(YSM_ConsoleGetPrompt(), -2, GL_BUF_SIZE);
 #endif
 #endif
-	}
+	} else 
+		PRINTF(VERBOSE_BASE, "\r%s", YSM_ConsoleGetPrompt());
 
 	fflush(stdout);
 }
@@ -237,7 +238,7 @@ YSM_ConsoleSetup(void)
  */
 
 void
-YSM_SendMessage( int32_t	r_uin,
+YSM_SendMessage( uin_t 		r_uin,
 		int8_t		*data,
 		int8_t		logflag,
 		YSM_SLAVE	*slave,
@@ -251,6 +252,7 @@ time_t		log_time;
 struct		tm *time_stamp;
 keyInstance	*crypt_key = NULL;
 
+
 	/* we save these marks to check if we need to free data
 	 * at the end of this function. */
 	oldmark1 = data;
@@ -338,9 +340,13 @@ keyInstance	*crypt_key = NULL;
 
 	/* what do we need to free? */
 	if (data != oldmark1) {
-		if (data != oldmark2 && oldmark2 != oldmark1) 
+		if (data != oldmark2 && oldmark2 != oldmark1) {
 			YSM_Free( oldmark2, __FILE__, __LINE__ );
+			oldmark2 = NULL;
+		}
+
 		YSM_Free( data, __FILE__, __LINE__ );
+		data = NULL;
 	}
 
 	if (slave != NULL) r_nick = slave->info.NickName;
@@ -453,7 +459,7 @@ int32_t YSM_ConsoleIsHotKey(int32_t key)
 		case '5':
 			PRINTF(VERBOSE_BASE, "\n");
 			PRINTF(VERBOSE_BASE, "%s\n", MSG_AFK_READ_MSG);
-			YSM_ReadLog(YSM_AFKFILENAME);
+			YSM_ReadLog(YSM_AFKFILENAME, 0);
 			break;
 
 #ifdef YSM_WITH_THREADS
@@ -651,6 +657,7 @@ int8_t	*aux = NULL;
 #ifdef HAVE_LIBREADLINE
 	/* dont you dare to leak on me! */
 	YSM_Free(retline, __FILE__, __LINE__);
+	retline = NULL;
 #endif
 }
 
@@ -733,7 +740,8 @@ void
 YSM_DoCommand( char *cmd )
 {
 int8_t		*argv[MAX_CMD_ARGS], found = FALSE;
-u_int32_t	argc = 0, x = 0;
+int32_t	argc = 0;
+u_int32_t x = 0;
 YSM_COMMAND	*node = plist_firstCOMMAND;
 
 	if (!strlen(cmd)) return;
@@ -827,7 +835,7 @@ YSM_ReadLongMessage(void)
 {
 static	int8_t msgdata[MAX_DATA_LEN + 1];
 int8_t	*retline = NULL, *tmp = NULL;
-int32_t	x = 0, y = 0;
+size_t	x = 0, y = 0;
 #ifndef HAVE_LIBREADLINE
 int8_t	*backup = NULL;
 #endif
@@ -878,16 +886,19 @@ int8_t	*aux = NULL;
 	
 			y = strlen(retline) + 1;	/* +1 of the \n */
 
-			/* if the input exceeds the maximum length,
-			 * resize it to the maximum. */
-			if ((x + y) > (int32_t)(sizeof(msgdata) - 1)) 
+			/* if the input exceeds the maximum length, resize it
+			 * to the maximum. no exploitable integer overflow here.
+			 */
+
+			if ((x + y) > (sizeof(msgdata) - 1)) 
 				y = MAX_DATA_LEN - x;
 
 			x += y;
 
 			/* if !y then we have no more space left.
-			 * Warn and return what we read so far */
-			if (y <= 0) {
+			 * Warn and return what we read so far 
+			 */
+			if (!y) {
 				PRINTF( VERBOSE_BASE,
 					"\nMaximum Data Reached. \a!\n");
 
@@ -901,13 +912,16 @@ int8_t	*aux = NULL;
 					backup,
 					sizeof(g_cmdstring));
 				YSM_Free(backup, __FILE__, __LINE__);
+				backup = NULL;
 #else
 #ifdef HAVE_LIBREADLINE
 				/* dont you dare to leak on me! */
 				YSM_Free(retline, __FILE__, __LINE__);
+				retline = NULL;
 #else
 				memcpy(gl_buf, backup, sizeof(gl_buf));
 				YSM_Free(backup, __FILE__, __LINE__);
+				backup = NULL;
 #endif
 #endif
 				return (tmp);
@@ -927,13 +941,16 @@ int8_t	*aux = NULL;
 					backup,
 					sizeof(g_cmdstring));
 				YSM_Free(backup, __FILE__, __LINE__);
+				backup = NULL;
 #else
 #ifdef HAVE_LIBREADLINE
 				/* dont you dare to leak on me! */
 				YSM_Free(retline, __FILE__, __LINE__);
+				retline = NULL;
 #else
 				memcpy(gl_buf, backup, sizeof(gl_buf));
 				YSM_Free(backup, __FILE__, __LINE__);
+				backup = NULL;
 #endif
 #endif
 					return (tmp);
@@ -949,13 +966,16 @@ int8_t	*aux = NULL;
 					backup,
 					sizeof(g_cmdstring));
 				YSM_Free(backup, __FILE__, __LINE__);
+				backup = NULL;
 #else
 #ifdef HAVE_LIBREADLINE
 				/* dont you dare to leak on me! */
 				YSM_Free(retline, __FILE__, __LINE__);
+				retline = NULL;
 #else
 				memcpy(gl_buf, backup, sizeof(gl_buf));
 				YSM_Free(backup, __FILE__, __LINE__);
+				backup = NULL;
 #endif
 #endif
 					return (NULL);
@@ -985,8 +1005,8 @@ int8_t	*aux = NULL;
 
 
 /* YSM_AlertUser()
- *	- Beeps and/or Flashes the Window.
- *	- by now flashing supported only in win32.
+ *	- Beeps
+ *	- configurable deconify & flashing window (win32/OS2)
  */
 
 static void
@@ -1001,17 +1021,14 @@ u_int32_t	x;
 		}
 	}
 
-#if defined(WIN32) || defined(OS2)
 	YSM_WindowAlert();
-#endif
-
 }
 
 static void
 YSM_PreIncoming( YSM_SLAVE	*contact,
 		int16_t		m_type,
 		int32_t		*m_len,
-		int32_t		r_uin,
+		uin_t		r_uin,
 		int8_t		**m_data,
 		int8_t		*r_nick,
 		u_int8_t	m_flags,
@@ -1143,33 +1160,33 @@ int	arg_index = 0, x = 0, y = 0;
 static void
 YSM_PostIncoming( YSM_SLAVE	*victim,
 		short	m_type,
-		int	r_uin,
+		uin_t	r_uin,
 		int	m_len,
 		char	*m_data,
 		char	*r_nick,
 		int	log_flag )
 {
+
+	int8_t fl_inchat = 0, fl_vinchat = 0, fl_inafk = 0;
+
 	if (m_type == YSM_MESSAGE_NORMAL) {
 
+		if (g_promptstatus.flags & FL_CHAT) fl_inchat = 1;
+		if (victim != NULL && (victim->flags & FL_CHAT)) fl_vinchat = 1;
+		if (g_promptstatus.flags & FL_AFKM) fl_inafk = 1;
+
 		/* only log normal messages */
-
-		if (log_flag 
-			|| YSM_SETTING_LOGALL 
-			|| (g_promptstatus.flags & FL_AFKM)
-			|| ((g_promptstatus.flags & FL_CHATM) 
-				&& victim && (victim->flags & FL_CHAT))) {
-
+		if ((log_flag || YSM_SETTING_LOGALL || fl_inafk || fl_inchat) && !fl_vinchat)
 			YSM_GenerateLogEntry(r_nick,
 					r_uin,
 					r_uin,
 					m_data,
 					m_len);
 
-			if (victim && ((g_promptstatus.flags & FL_AFKM) 
-			|| !(victim->flags & FL_CHAT))) {
+			if ((g_promptstatus.flags & FL_AFKM) || 
+			(victim != NULL && !(victim->flags & FL_CHAT))) {
 				YSM_AFKCount++;
 			}
-		}
 
 		/* do we have to send messages? either AFK, CHAT or FORWARD? */
 
@@ -1233,7 +1250,7 @@ YSM_PostIncoming( YSM_SLAVE	*victim,
 
 void
 YSM_PreOutgoing( YSM_SLAVE	*victim,
-		int32_t		r_uin,
+		uin_t		r_uin,
 		int8_t		**m_data,
 		int32_t		*m_len,
 		int8_t		logflag )
@@ -1241,8 +1258,9 @@ YSM_PreOutgoing( YSM_SLAVE	*victim,
 int8_t	*data_conv = NULL;
 u_int8_t m_flags = 0;
 
-	if ((YSM_SETTING_LOGALL || logflag) 
-		&& (!(g_promptstatus.flags & FL_AFKM)))
+
+	if ((victim == NULL && (YSM_SETTING_LOGALL || logflag)) 
+		|| ((YSM_SETTING_LOGALL || logflag) && (victim && !(victim->flags & FL_CHAT))))
 		YSM_GenerateLogEntry(
 				"y0u",
 				YSM_USER.Uin,
@@ -1261,7 +1279,7 @@ u_int8_t m_flags = 0;
 }
 
 void
-YSM_PostOutgoing( int32_t	r_uin,
+YSM_PostOutgoing( uin_t		r_uin,
 		int8_t		*r_nick,
 		int32_t		m_len,
 		int8_t		*m_data )
@@ -1273,7 +1291,7 @@ YSM_PostOutgoing( int32_t	r_uin,
 
 void
 YSM_DisplayMsg( int16_t		m_type,
-		int32_t		r_uin,
+		uin_t		r_uin,
 		u_int16_t	r_status,
 		int32_t		m_len,
 		int8_t		*m_data,
@@ -1288,7 +1306,10 @@ keyInstance	*crypt_key = NULL;
 time_t		log_time;
 struct tm	*time_stamp;
 int		x = 0;
+int8_t		*msgCol = YSM_SETTING_COLOR_MESSAGE;
 YSM_SLAVE	*contact = NULL;
+int8_t		*old_m_data = NULL;
+int		free_m_data = 0;
 
 
 	YSM_WriteStatus(r_status, status_string);
@@ -1297,6 +1318,7 @@ YSM_SLAVE	*contact = NULL;
 		r_nick = "NOT a SLAVE";
 	} else {
 		contact = YSM_QuerySlaves(SLAVE_NAME, r_nick, 0, 0);
+	 	if (contact->Color!=NULL) msgCol = contact->Color;
 	}
 
 #if defined(YSM_WITH_THREADS)
@@ -1304,6 +1326,7 @@ YSM_SLAVE	*contact = NULL;
 	while ((g_promptstatus.flags & FL_BUSYDISPLAY) || (g_promptstatus.flags & FL_COMFORTABLEM)) YSM_Thread_Sleep( 0, 500 );
 #endif
 
+	old_m_data = m_data;
 	YSM_PreIncoming( contact,
 			m_type,
 			&m_len,
@@ -1313,6 +1336,7 @@ YSM_SLAVE	*contact = NULL;
 			m_flags,
 			&crypt_key );
 
+	free_m_data = (m_data != old_m_data);
 	log_time = time(NULL);
 
 	/* are we in CHAT MODE ? we dont print messages which don't belong
@@ -1360,39 +1384,6 @@ YSM_SLAVE	*contact = NULL;
 
 		} else {
 		
-		switch( YSM_SETTING_MSG_KIND ) {
-		
-		case YSM_MSG_KIND_ONELINE:
-		
-#ifdef COMPACT_DISPLAY
-		/* chop first char if it's newline '*' tells we chopped */
-		if (m_data[0] == '\n')
-			m_data[0] = '*';
-		else if (m_data[0] == '\r')
-			m_data[0] = '*';
-		if (m_data[1] == '\n')
-			m_data[1] = '*';
-#endif
-		PRINTF( VERBOSE_BASE, "\r--> %s",ctime(&log_time));
-		PRINTF( VERBOSE_BASE,
-			"%s<%s%s | %d | %s%s>%s[len:%d] %s%s%s\n",
-			BLUE, NORMAL, r_nick, r_uin, status_string, BLUE,
-			NORMAL, m_len, YSM_SETTING_COLOR_MESSAGE, m_data, NORMAL );
-			break;
-
-		case YSM_MSG_KIND_VERBOSE:
-
-		PRINTF( VERBOSE_BASE, "\r--> %s",ctime(&log_time));
-		PRINTF( VERBOSE_BASE,
-			"%s - SLAVE NAME: %s - UIN: %d\n"
-		    	"<DATA %s [len:%d]>: %s\n",
-			MSG_INCOMING_MESSAGE, r_nick, r_uin,
-			(crypt_key != NULL) 
-			? "-Encrypted-" : "", m_len, m_data);
-			break;
-
-		case YSM_MSG_KIND_IRC:
-		{
 			time_stamp = localtime( &log_time );
 			strftime( time_string, 9, "%H:%M:%S", time_stamp );
 
@@ -1406,7 +1397,7 @@ YSM_SLAVE	*contact = NULL;
 					(crypt_key != NULL) ? "<*" : "<",
 					r_nick,
 					(crypt_key != NULL) ? "*>" : ">",
-					YSM_SETTING_COLOR_MESSAGE,
+					msgCol,
 					(strchr(m_data,'\n') != NULL) 
 					? "\n" : "",
 					m_data );
@@ -1421,20 +1412,12 @@ YSM_SLAVE	*contact = NULL;
 					(crypt_key != NULL) ? "<*" : "<",
 					r_uin,
 					(crypt_key != NULL) ? "*>" : ">",
-					YSM_SETTING_COLOR_MESSAGE,
+					msgCol,
 					(strchr(m_data,'\n') != NULL) 
 					? "\n" : "",
 					m_data );
 			}
 
-			break;
-		}	/* case */
-
-		default:
-				break;
-
-		}	/* switch */
-
 		}	/* else */
 
 		break;
@@ -1559,6 +1542,11 @@ displaymsg_exit:
 			m_data,
 			r_nick,
 			log_flag );
+
+	if (free_m_data) {
+		YSM_Free(m_data, __FILE__, __LINE__);
+		m_data = NULL;
+	}
 }
 
 int32_t
@@ -1605,7 +1593,7 @@ YSM_COMMAND	*node = plist_firstCOMMAND;
 			break;
 		} else if(node->cmd_alias != NULL) {
 			if(!memcmp(node->cmd_alias, incomplete, strlen(incomplete))) {
-				match = node->cmd_name;
+				match = node->cmd_alias;
 				break;
 			}
 		}
@@ -1626,8 +1614,11 @@ YSM_COMMAND	*node = plist_firstCOMMAND;
 		tmpbuf[sizeof(tmpbuf) - 1] = 0x00;
 
 		/* go back to where the command starts. */
+#if defined(YSM_WITH_THREADS) && defined(HAVE_LIBREADLINE)
+		(*pos) = 0;
+#else
 		(*pos) -= strlen(incomplete);
-
+#endif
 
 #if defined(YSM_WITH_THREADS) && defined(HAVE_LIBREADLINE)
 	/* for readline its different because the buffer is allocated
