@@ -1,4 +1,4 @@
-/*	$Id: YSM_Setup.c,v 1.137 2004/07/10 19:21:34 rad2k Exp $	*/
+/*	$Id: YSM_Setup.c,v 1.147 2005/12/26 23:44:58 rad2k Exp $	*/
 /*
 -======================== ysmICQ client ============================-
 		Having fun with a boring Protocol
@@ -26,7 +26,7 @@ For Contact information read the AUTHORS file.
 */
 
 #include "YSM.h"
-__RCSID("$Id: YSM_Setup.c,v 1.137 2004/07/10 19:21:34 rad2k Exp $");
+__RCSID("$Id: YSM_Setup.c,v 1.147 2005/12/26 23:44:58 rad2k Exp $");
 
 #include "YSM_Lists.h"
 #include "YSM_Setup.h"
@@ -195,7 +195,10 @@ int32_t	homepsize = 0, homep2size = 0;
 		sizeof(YSM_cfgfile) - strlen(YSM_cfgfile) - 1);
 
 	YSM_Free(homep, __FILE__, __LINE__);
+	homep = NULL;
+
 	YSM_Free(homep2, __FILE__, __LINE__);
+	homep2 = NULL;
 }
 
 
@@ -462,11 +465,6 @@ int8_t	YSM_CFGEND = FALSE, buf[MAX_PATH], *auxb = NULL, *aux = NULL;
 							~STATUS_FLBIRTHDAY;
 			}
 
-			else if(!strcasecmp(aux,"MSG_KIND")) {
-				if ((aux=strtok(NULL, " \n\t")) != NULL)
-					YSM_SETTING_MSG_KIND = atoi(aux);
-			}
-
 			else if(!strcasecmp(aux,"VERBOSE")) {
 				if ((aux=strtok(NULL, " \n\t")) != NULL)
 					YSM_SETTING_VERBOSE = atoi(aux);
@@ -692,9 +690,18 @@ int	tries = 0;
 
 	PRINTF( VERBOSE_BASE, "\nRegister a new ICQ Number.\n");
 	PRINTF( VERBOSE_BASE, 
-		"You will be asked for your desired password twice.\n");
+		"Unfortunately, registering an ICQ number with ysmICQ is no longer possible.\n");
+
+	PRINTF(VERBOSE_BASE, "Please head to http://www.icq.com/register/ to register a UIN.\n");
+
+	PRINTF(VERBOSE_BASE, "After you have registered a UIN, come back and type it in again.\n");
+
+		PRINTF( VERBOSE_BASE,
+		"\nYour " RED "UIN" NORMAL "#: " );
+
+	YSM_fgets(YSM_tmpb, MAX_UIN_LEN, 0);
+	YSM_USER.Uin = atoi(YSM_tmpb);
 
-	PRINTF(VERBOSE_BASE, "The Password must be 8 chars max.\n");
 
 	} else	{
 		PRINTF(VERBOSE_BASE,
@@ -720,9 +727,6 @@ int	tries = 0;
 
 	if (!strcasecmp( YSM_USER.password , YSM_tmpa )) {
 	
-		if (YSM_USER.Uin == 0)
-			YSM_UINRegistrationRequest(YSM_USER.password);
-
 		YSM_SaveConfig();
 
 		PRINTF(VERBOSE_BASE,
@@ -902,15 +906,14 @@ FILE	*fd;
 		CFGWRITE(fd, "# Let everyone know its my birthday!");
 		CFGWRITE(fd, "MYBIRTHDAY>0\n");
 
-#if defined(WIN32) || defined(OS2)
 		CFGWRITE(fd,
-	"# WINALERT is a Console setting for incoming messages.\n"
-	"# If set to \"1\" it will only popup.\n"
-	"# If set to \"2\" it will only blink.\n"
-	"# If set to \"3\" it will popup and blink.");
+	"# WINALERT will alert your console window on incoming messages.\n"
+	"# If set to \"0\" windows alerts will be disabled.\n"
+	"# If set to \"1\" it will only popup (unix/win32/os2).\n"
+	"# If set to \"2\" it will only blink (win32 and OS2 only).\n"
+	"# If set to \"3\" it will popup and blink (only blinks in win32 and OS2).");
 		CFGWRITE(fd,"WINALERT>%d\n", YSM_SETTING_WINALERT);
 
-#endif
 #ifdef WIN32
 		CFGWRITE(fd,
 	"# WINHOTKEY is a CTRL+ALT+key combination in charge of\n"
@@ -971,7 +974,7 @@ FILE	*fd;
 
 		CFGWRITE(fd, 
 		"# Colors configuration. Available colors are:\n"
-		"# BLACK RED GREEN BROWN BLUE MAGENTA CYAN WHITE\n"
+		"# BLACK RED GREEN BROWN BLUE MAGENTA CYAN GRAY WHITE TERMINAL_DEFAULT\n"
 		"# and all mentioned colors in bright as in BRIGHT_colorname.\n"
 		"# You can either use defaults or change them below.");
 		
@@ -989,9 +992,6 @@ FILE	*fd;
 		CFGWRITE(fd,"# FishGUI - Hide ysm console?");
 		CFGWRITE(fd,"FISHGUI_HIDE_CONSOLE>0\n");
 
-		CFGWRITE(fd,"# Msg Kind types are 0(ONELINE) 1(VERBOSE) 2(IRC)");
-		CFGWRITE(fd,"MSG_KIND>%d\n", YSM_SETTING_MSG_KIND);
-
 		CFGWRITE(fd, "# Verbose level. Add or remove output information.\n"
 			"# Normal output -> 5\n"
 			"# - Remove status changes -> 0\n"
@@ -1023,6 +1023,9 @@ FILE	*fd;
 		CFGWRITE(fd,"# Russian Generic (for Unix) are "
 			"TRANS: CP1251 LOCAL: KOI8-R");
 
+		CFGWRITE(fd,"# Russian Generic (for Windows) are "
+			"TRANS: 1251 LOCAL: 866");
+
 		CFGWRITE(fd,"# Windows users have a list of supported charsets at:"
 		"\n# HKEY_LOCAL_MACHINE\\SYSTEM\\CurrentControlSet\\Control\\NIs"
 		"\\CodePage");
@@ -1044,116 +1047,104 @@ FILE	*fd;
 }	
 	
 
-
 	
 void
 YSM_ReadSlaves( FILE *fd )
 {
+int8_t	YSM_tmpbuf[MAX_PATH], *aux = NULL;
+int8_t	*auxnick = NULL, *auxuin = NULL, *auxkey = NULL, *auxflags = NULL;
+int8_t	*auxcol = NULL;
+int8_t	*next = NULL;
+int	field = 0;
 
-char	YSM_tmpbuf[MAX_PATH], *aux = NULL;
-char	*auxnick = NULL,*auxuin = NULL,*auxkey = NULL, *auxflags = NULL;
-short	YSM_MORESLAVES=TRUE;
+	if (YSM_IsInvalidPtr(fd))
+		return;
 
+	/* find start of SLAVES section */
+	while (memset(YSM_tmpbuf, '\0', sizeof(YSM_tmpbuf))
+	&& fgets(YSM_tmpbuf, sizeof(YSM_tmpbuf)-1, fd)!= NULL) {
 
-	while(YSM_MORESLAVES) {
-
-	memset(YSM_tmpbuf,'\0',MAX_PATH);
-
-	if(fgets(YSM_tmpbuf,sizeof(YSM_tmpbuf)-1,fd)==NULL) {
-		YSM_MORESLAVES = FALSE;
-		continue;
-       	}
-
-	if ((YSM_tmpbuf[0]!='#') && (YSM_tmpbuf[0] != 0) 
-	&& (YSM_tmpbuf[0] != '\n')) {
-
-		auxnick = strtok(YSM_tmpbuf,"\n");
-
-        	if (!strcasecmp( auxnick, SLAVES_TAG )) {
-               		memset(YSM_tmpbuf,'\0',MAX_PATH);
-                
-			while(YSM_MORESLAVES) {
-
-		                if ( fgets( YSM_tmpbuf, sizeof(YSM_tmpbuf)-1,
-							fd ) == NULL) {
-                        
-					YSM_MORESLAVES = FALSE;
-                        		continue;
-                        	}
-
-                		if ((YSM_tmpbuf[0] == '#')
-					|| (YSM_tmpbuf[0] == '\0')
-					|| (YSM_tmpbuf[0] == '\n')) {
-
-					continue;
-				}
-			
-                       		auxnick = strtok(YSM_tmpbuf,":");
-                       		if (auxnick) auxuin = strtok(NULL, ":");
-				if (auxuin) {
-					auxkey = auxuin+strlen(auxuin)+1;
-					if (auxkey != NULL)
-						aux = strchr(auxkey,':');
-					else
-						aux = NULL;
-
-					if (aux != NULL) {
-						/* flags field exists */
-						*aux = 0x00;
-						aux = auxkey+strlen(auxkey)+1;
-						auxflags = strtok(aux," \n\t");
-					}
-					else {
-						auxkey = strtok(NULL," \n\t");
-						auxflags = NULL;
-					}
-				}
-
-				if (auxkey != NULL && strlen(auxkey) < 1)
-					auxkey = NULL;
-					
-				if(auxnick == NULL || auxuin == NULL) {
-				/* No nick/uin/flags specified, Weird! */
-				/* Lucky slave! */
-				/* Continue with the next slave */
-					continue;
-				}
-
-				/* Check we have a valid line here */
-				/* 4 number uins must be fake or something. */
-				if (strlen(auxnick) < 1 || strlen(auxuin) < 5 )
-				{
-				/* Go get a new slave, dammit */
-				/* Slaves nowadays need new brands */
-					continue;
-				}
-			
-				YSM_AddSlavetoList( auxnick,
-						atoi(auxuin),
-						auxflags,
-						auxkey,
-						0,
-						0,
-						0,
-						0 );
-               		}
+		if (!strcasecmp( YSM_trim(YSM_tmpbuf), SLAVES_TAG )) {
+			break;
 		}
-        } 
-
 	}
 
-	PRINTF( VERBOSE_MOREDATA,
-		"%s%d]" NORMAL "\n",
-		MSG_READ_SLAVES,
-		List_amountSLAVE );
-}
+	/* parse slaves */
+	while (memset(YSM_tmpbuf, '\0', sizeof(YSM_tmpbuf))
+	&& fgets(YSM_tmpbuf, sizeof(YSM_tmpbuf)-1, fd) != NULL) {
 
+		YSM_trim(YSM_tmpbuf);
+		if (YSM_tmpbuf[0]=='#' ) continue; /* ignore comments */
+
+		aux = YSM_tmpbuf;
+		next = strchr(aux, ':');
+		if (next) *next++ = '\0';
+
+		auxnick = auxuin = auxkey = auxflags = auxcol = NULL;
+		field = 0;
+
+		while (aux != NULL) {
+			YSM_trim(aux);
+
+			switch (field++) {
+
+				case 0 : /* Nick */
+					if (*aux != '\0') auxnick = aux;
+					break;
+
+				case 1 : /* UIN */
+					if (*aux != '\0') auxuin = aux;
+					break;
+
+				case 2 : /* Key */
+					if (*aux != '\0') auxkey = aux;
+					break;
+
+				case 3: /* Flags */
+					if (*aux != '\0') auxflags = aux;
+					break;
+
+				case 4: /* Color */
+					if (*aux != '\0') auxcol = YSM_GetColorByName(aux);
+					break;
+
+				default: /* Trailing garbage */
+					break;
+			}
+
+			aux = next;
+			if (aux) next = strchr(aux, ':');
+			if (next) *next++ = '\0';
+		}
+
+		if (auxnick == NULL	/* No name */
+		|| auxuin == NULL	/* No UIN */
+		|| strlen(auxuin) < 5)	/* Invalid UIN */
+			continue;
+
+                YSM_AddSlavetoList( auxnick,
+                                atoi(auxuin),
+                                auxflags,
+                                auxkey,
+				auxcol,
+                                0,
+                                0,
+                                0,
+                                0 );
+ 
+        }
+ 
+        PRINTF( VERBOSE_MOREDATA,
+                "%s%d]" NORMAL "\n",
+                MSG_READ_SLAVES,
+                List_amountSLAVE );
+}
 
 YSM_SLAVE *
 YSM_QuerySlaves( unsigned short TYPE,
-	unsigned char	*Extra,
-	unsigned int	Uin,
-	unsigned int	reqid ) 
+		unsigned char	*Extra,
+		uin_t		Uin,
+		unsigned int	reqid ) 
 {
 
 u_int32_t	x;
@@ -1183,7 +1174,7 @@ YSM_SLAVE	*node = plist_firstSLAVE;
 }
 
 void
-YSM_AddSlave( char *Name, int Uin ) 
+YSM_AddSlave( char *Name, uin_t Uin ) 
 {
 YSM_SLAVE *result = NULL;
 
@@ -1191,6 +1182,7 @@ YSM_SLAVE *result = NULL;
 				Uin,
 				NULL,
 				NULL,
+				NULL,
 				0,
 				0,
 				0,
@@ -1244,7 +1236,7 @@ u_int32_t	x = 0;
 			fprintf( YSM_tmp, 
 				"%s:%d:",
 				victim->info.NickName,
-				victim->Uin );
+				(int)victim->Uin );
 
 			/* Fill Key */
 			if (!YSM_KeyEmpty( victim->crypto.strkey )) {
@@ -1509,14 +1501,19 @@ YSM_AFKMode(u_int8_t turnflag)
 	}
 }
 
+/* logtype:	0 => AFK
+ *		1 => Slave
+ */
+
 void
-YSM_ReadLog(char *FileName)
+YSM_ReadLog(char *FileName, int logtype)
 {
 FILE	*logfile = NULL;
-int8_t 	*rfilename = NULL, rwho[MAX_NICK_LEN], rmsg[MAX_DATA_LEN+1];
+int8_t 	*rfilename = NULL;
 int32_t	mnum = 1, tnum = 1, snum = 0, size = 0;
 int8_t	q, *rtime = NULL, *aux = NULL, *auxb = NULL;
-int8_t	buff[ MAX_DATA_LEN+MAX_NICK_LEN+MAX_UIN_LEN+2 ];
+/* 64 bytes extra for YSM_LOG_SEPARATORs, date, etc */
+int8_t	buff[ MAX_DATA_LEN+MAX_NICK_LEN+MAX_UIN_LEN+64 ];
 
 	if (FileName == NULL) return;
 	
@@ -1530,6 +1527,7 @@ int8_t	buff[ MAX_DATA_LEN+MAX_NICK_LEN+MAX_UIN_LEN+2 ];
 			"\nNo Messages Found :: filename not found.\n");
 
 		YSM_Free(rfilename, __FILE__, __LINE__);
+		rfilename = NULL;
 		return;
 	}
 
@@ -1566,8 +1564,10 @@ int8_t	buff[ MAX_DATA_LEN+MAX_NICK_LEN+MAX_UIN_LEN+2 ];
 					}
 
 					YSM_Free(rfilename, __FILE__, __LINE__);
+					rfilename = NULL;
+
+					if (logtype == 0) YSM_AFKCount = 0;
 
-					YSM_AFKCount = 0;
 					return;
 	
 					break;
@@ -1589,55 +1589,26 @@ int8_t	buff[ MAX_DATA_LEN+MAX_NICK_LEN+MAX_UIN_LEN+2 ];
 						__FILE__,
 						__LINE__);
 
+					rfilename = NULL;
+
 					return;
 					break;
 			}
 
 		}
 
-		if(strlen(buff) < 2)
-			continue;
-
-
-		memset(rwho ,'\0', MAX_NICK_LEN);
-		memset(rmsg ,'\0', MAX_DATA_LEN+1);
 
+		YSM_trim(buff);
 		
-		aux = strstr(buff, YSM_LOG_SEPARATOR);
-		if (aux != NULL)
-		{	
-			memcpy(&rwho[0], &buff[0], aux - &buff[0]);
-			aux += strlen(YSM_LOG_SEPARATOR);
-		
-			auxb = strstr(aux, YSM_LOG_SEPARATOR);
-			if (auxb != NULL)
-			{
-				memcpy(&rmsg[0], aux, auxb - aux);
-				auxb += strlen(YSM_LOG_SEPARATOR);
-			}
-
-			rtime = strtok(auxb, "");
-
-		} else {
+		if (*buff != '\0' && !YSM_DisplayLogEntry(buff, tnum)) {
 			g_promptstatus.flags &= ~FL_BUSYDISPLAY;
-			PRINTF(VERBOSE_BASE,"\nreadlog :: parsing error.");
+			PRINTF(VERBOSE_BASE,"\nreadlog :: parsing error.\n");
 			YSM_Free(rfilename, __FILE__, __LINE__);
+			rfilename = NULL;
 			YSM_fclose(logfile);
 			return;
 		}
 	
-		g_promptstatus.flags &= ~FL_BUSYDISPLAY;
-		if (rwho) {
-			PRINTF(VERBOSE_BASE,
-				"\n"WHITE"Msg #%i:" NORMAL" - From: %s",
-							tnum, rwho );
-		}
-		if(rtime)
-			PRINTF(VERBOSE_BASE, "\nTime: %s", rtime );
-		if(rmsg)
-			PRINTF(VERBOSE_BASE, "Data: %s\n", rmsg );
-
-		g_promptstatus.flags |= FL_BUSYDISPLAY;
 		mnum ++; tnum ++;
 
 		if (snum > 0) snum--;
@@ -1651,6 +1622,38 @@ int8_t	buff[ MAX_DATA_LEN+MAX_NICK_LEN+MAX_UIN_LEN+2 ];
 
 	YSM_fclose( logfile );
 	YSM_Free(rfilename, __FILE__, __LINE__);
+	rfilename = NULL;
+}
+
+/* YSM_DisplayLogEntry - returns 0 if a parsing error occurs.
+ */
+
+int
+YSM_DisplayLogEntry(int8_t *buf, int32_t messageNum)
+{
+char*	part[3+1];	/* 1 more than 3 to distinguish 3 from >3 fields */
+ssize_t	fields;
+
+	fields = YSM_tokenize( buf,
+			YSM_LOG_SEPARATOR,
+			part,
+			NUM_ELEM_ARR(part));
+
+	if (fields != 3)
+		return 0;
+
+	g_promptstatus.flags &= ~FL_BUSYDISPLAY;
+
+	PRINTF(VERBOSE_BASE,
+		"\n" WHITE "Msg #%i:" NORMAL " - From: %s",
+		messageNum,
+		part[0]);
+
+	PRINTF(VERBOSE_BASE, "\nTime: %s", part[2]);
+	PRINTF(VERBOSE_BASE, "\nData: %s\n", part[1]);
+
+	g_promptstatus.flags |= FL_BUSYDISPLAY;
+	return 1;
 }
 
 /* The YSM_VersionCheck function is used to check the local client 
@@ -1937,7 +1940,7 @@ int	x = 0;
 			YSM_WriteStatus(YSM_USER.status, tmp);
 			setenv("YSMSTATUS", tmp, 1);
 			/* our own UIN */	
-			snprintf(tmp, sizeof(tmp)-1, "%d", YSM_USER.Uin);
+			snprintf(tmp, sizeof(tmp)-1, "%d", (int)YSM_USER.Uin);
 			tmp[sizeof(tmp)-1] = 0x00;
 			setenv("YSMUIN", tmp, 1);
 #endif		
