@@ -1,4 +1,4 @@
-/*	$Id: YSM_ToolBox.c,v 1.82 2004/07/05 01:00:51 rad2k Exp $	*/
+/*	$Id: YSM_ToolBox.c,v 1.89 2005/09/04 01:36:48 rad2k Exp $	*/
 /*
 -======================== ysmICQ client ============================-
 		Having fun with a boring Protocol
@@ -26,7 +26,7 @@ For Contact information read the AUTHORS file.
 */
 
 #include "YSM.h"
-__RCSID("$Id: YSM_ToolBox.c,v 1.82 2004/07/05 01:00:51 rad2k Exp $");
+__RCSID("$Id: YSM_ToolBox.c,v 1.89 2005/09/04 01:36:48 rad2k Exp $");
 
 #include "YSM_ToolBox.h"
 #include "YSM_Wrappers.h"
@@ -57,7 +57,7 @@ HANDLE g_hSuicideEvent, g_hThreadDiedEvent;
 
 void
 YSM_Event( int8_t	event_t,
-	int32_t		r_uin,
+	uin_t		r_uin,
 	int8_t		*r_nick,
 	int32_t		m_len,
 	int8_t		*m_data,
@@ -66,9 +66,9 @@ YSM_Event( int8_t	event_t,
 int8_t	uinstring[MAX_UIN_LEN+1], length[10];
 int8_t	*event_action = NULL, *event_sound = NULL, event_senable = 0;
 
-	snprintf(uinstring, sizeof(uinstring), "%d", r_uin);
+	snprintf(uinstring, sizeof(uinstring), "%d", (int)r_uin);
 	uinstring[sizeof(uinstring) - 1] = 0x00;
-	snprintf(length, sizeof(length), "%d", m_len);
+	snprintf(length, sizeof(length), "%d", (int)m_len);
 	length[sizeof(length) - 1] = 0x00;
 
 	switch (event_t) {
@@ -155,6 +155,7 @@ struct stat	filestat;
 	if (stat(path, &filestat)) {
 		/* file not found. return an error */
 		YSM_Free( path, __FILE__, __LINE__ );
+		path = NULL;
 		return -1;
 	}
 
@@ -175,9 +176,10 @@ struct stat	filestat;
 	/* play the god damn sound at once! */
 	YSM_ExecuteLine( path2, NULL, NULL, NULL, NULL );
 	YSM_Free( path2, __FILE__, __LINE__ );
+	path2 = NULL;
 #endif
 	YSM_Free( path, __FILE__, __LINE__ );
-
+	path = NULL;
 	return 0;
 }
 
@@ -382,9 +384,12 @@ YSM_WriteStatus(u_int16_t status, int8_t *buf)
 }
 
 int8_t *
-YSM_GetColorStatus(int8_t *status)
+YSM_GetColorStatus(int8_t *status, int8_t* override)
 {
-	if (status == NULL)
+	if (override)
+		return override;
+
+	if (YSM_IsInvalidPtr(status))
 		return GREEN;
 
 	if(!strcmp(status, "ONLINE"))		return GREEN;
@@ -414,7 +419,9 @@ YSM_GetColorByName(int8_t *color)
 	if(!strcasecmp(color, "MAGENTA"))	return MAGENTA;
 	if(!strcasecmp(color, "WHITE"))		return WHITE;
 	if(!strcasecmp(color, "CYAN"))		return CYAN;
+	if(!strcasecmp(color, "GRAY"))		return GRAY;
 	if(!strcasecmp(color, "NORMAL"))	return NORMAL;
+	if(!strcasecmp(color, "TERMINAL_DEFAULT"))	return TERMINAL_DEFAULT;
 	if(!strcasecmp(color, "BRIGHT_BLACK"))	return BRIGHT_BLACK;
 	if(!strcasecmp(color, "BRIGHT_BLUE"))	return BRIGHT_BLUE;	
 	if(!strcasecmp(color, "BRIGHT_RED"))	return BRIGHT_RED;	
@@ -422,15 +429,17 @@ YSM_GetColorByName(int8_t *color)
 	if(!strcasecmp(color, "BRIGHT_BROWN"))	return BRIGHT_BROWN;	
 	if(!strcasecmp(color, "BRIGHT_MAGENTA"))	return BRIGHT_MAGENTA;	
 	if(!strcasecmp(color, "BRIGHT_CYAN"))	return BRIGHT_CYAN;	
+	if(!strcasecmp(color, "BRIGHT_GRAY"))	return BRIGHT_GRAY;
 	if(!strcasecmp(color, "BRIGHT_WHITE"))	return BRIGHT_WHITE;	
+	if(!strcasecmp(color, "BRIGHT_TERMINAL_DEFAULT"))	return BRIGHT_TERMINAL_DEFAULT;
 	
 	return GREEN;
 }
 
 void
 YSM_GenerateLogEntry( int8_t	*nick,
-		int32_t		UINa,
-		int32_t		UINb,
+		uin_t		uinA,
+		uin_t		uinB,
 		int8_t		*message,
 		int32_t		mlen )
 {
@@ -438,6 +447,8 @@ char	log_name[MAX_PATH], log_msg[MAX_DATA_LEN+1], *log_data, *aux = NULL;
 time_t	log_time;
 int	log_len = 0, a = 0;
 
+	if (uinA <= 0 || uinB <= 0)
+		return;
 
 	log_time = time(NULL);
 
@@ -451,7 +462,7 @@ int	log_len = 0, a = 0;
 			(g_promptstatus.flags & FL_CHATM)) {
 		snprintf(log_name, MAX_PATH, "%s", YSM_AFKFILENAME);
 	} else
-		snprintf(log_name, MAX_PATH, "%d", UINb);
+		snprintf(log_name, MAX_PATH, "%d", (int)uinB);
 
 	log_name[sizeof(log_name) - 1] = 0x00;
 
@@ -473,7 +484,7 @@ int	log_len = 0, a = 0;
 		log_len,
 		"<%s|%d> %s %s %s %s",
 		nick,
-		UINa,
+		(int)uinA,
 		YSM_LOG_SEPARATOR,
 		log_msg,
 		YSM_LOG_SEPARATOR,
@@ -484,6 +495,7 @@ int	log_len = 0, a = 0;
 	/* theres a newline by ctime at the end */
 	YSM_DumpLogFile(log_name,log_data);
 	YSM_Free(log_data, __FILE__, __LINE__);
+	log_data = NULL;
 }
 
 int32_t
@@ -519,6 +531,7 @@ int32_t	size = 0;
 	fd = YSM_fopen( path, attr );
 	
 	YSM_Free( path, __FILE__, __LINE__ );
+	path = NULL;
 	
 	return fd;
 }
@@ -597,6 +610,39 @@ int8_t	buf[MAX_PATH];
 
 }
 
+/*
+  Removes leading and trailing whitespace from str if str is non-NULL.
+  Returns: str, if str != NULL
+           NULL, if str == NULL
+*/
+int8_t *
+YSM_trim( int8_t *str )
+{
+int8_t *str2 = NULL;
+int8_t *str3 = NULL;
+
+	if (YSM_IsInvalidPtr(str))
+		return NULL;
+
+	str2 = str;
+
+	while (isspace(*str2)) ++str2;
+
+	str3 = str2 + strlen(str2);
+	while (str3!=str2) if (!isspace(*(--str3))) break;
+
+	if (*str2=='\0')
+		*str='\0';
+	else {
+		int8_t *str4 = str;
+		++str3;
+		while (str2!=str3) *str4++ = *str2++;
+		*str4 = '\0';
+	}
+
+	return str;
+}
+
 void
 YSM_Print_Uptime( void )
 {
@@ -1133,3 +1179,45 @@ int32_t	argc = 0, x = 0;
 	if (fd == NULL) return;
 	YSM_fclose(fd);
 }
+
+/*
+  The string str is split into tokens stored in array arr of maximum length
+  count.
+  Tokens are delimited by sep.
+  Occurrences of sep are not treated as part of any token.
+  The number of tokens in str is the number of occurrences of sep PLUS 1.
+  If str starts with sep, the first token is an empty string. If str ends with
+  sep, the last token is an empty string.
+
+  All token string pointers point directly into str which is destroyed
+  while tokenizing.
+
+  Returns: -1 if str, sep or arr is NULL or if count<0 or if strlen(sep)==0
+           the number of tokens written to arr (never more than count)
+*/
+
+ssize_t
+YSM_tokenize(char* str, const char* sep, char** arr, ssize_t count)
+{
+ssize_t ret = 0, len = -1;
+
+	if (sep != NULL) 
+		len = strlen(sep);
+
+	if (str == NULL || sep == NULL || arr == NULL || len <= 0 || count < 0)
+		return -1;
+
+	while (count>0) {
+		*arr = str;
+		--count;
+		++ret;
+		++arr;
+
+		str = strstr(str, sep);
+		if (str == NULL) break;
+		*str='\0';
+		str += len;
+	}
+
+	return ret;
+}
